﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace SpreadTest3
{
    public abstract class ActionThreadWithCache<CAction>
    {
        /// <summary>
        /// 默认线程
        /// </summary>
        private Thread _PThread = null;
        /// <summary>
        /// 线程是否在运行
        /// </summary>
        private bool _BIsRunning = false;

        /// <summary>
        /// 操作实体类的队列
        /// </summary>
        private List<CAction> _LstCActionEs = null;

        /// <summary>
        /// 线程队列锁
        /// </summary>
        private object _ObjListLock = new object();

        /// <summary>
        /// 本批次的任务总数
        /// </summary>
        private int _CntSumCAction = 0;

        /// <summary>
        /// 剩余任务数
        /// </summary>
        private int _CntRemainderCAction = 0;

        /// <summary>
        /// 本批次执行到任务的位置
        /// </summary>
        private int _IntCurrentCAction = 0;

        #region 消息类信息
        /// <summary>
        /// 用户看到的消息
        /// </summary>
        protected StringBuilder _UserMsg = new StringBuilder();

        /// <summary>
        /// 正常消息
        /// </summary>
        protected StringBuilder _NormalMsg = new StringBuilder();

        /// <summary>
        /// 错误信息
        /// </summary>
        protected StringBuilder _ErrorMsg = new StringBuilder();
        #endregion 消息类信息

        /// <summary>
        /// 模块名称
        /// </summary>
        public string ModuleName= string.Empty;

        /// <summary>
        /// 批量处理的数据
        /// </summary>
        public int CntBatchProcess = 1;

        public void SetValue(int cntBatchProcess, string strModuleName)
        {
            CntBatchProcess = cntBatchProcess;
            ModuleName = strModuleName;
        }

        /// <summary>
        /// 将文件信息增加到线程队列中
        /// </summary>
        /// <param name="cActionE"></param>
        public void AppendRangeCAction(CAction cActionE)
        {
            if (cActionE == null)
            {
                return;
            }
            try
            {
                lock (_ObjListLock)
                {
                    if (_LstCActionEs == null)
                    {
                        _LstCActionEs = new List<CAction>();
                    }
                    _LstCActionEs.Add(cActionE);
                    _CntSumCAction++;
                    _CntRemainderCAction++;

                    //启动线程；
                    StartThread();
                }
            }
            catch (Exception ex)
            {
                _ErrorMsg.AppendLine("Message：" + ex.Message);
                _ErrorMsg.AppendLine("StackTrace：" + ex.StackTrace);
                if (ex.InnerException != null)
                {
                    _ErrorMsg.AppendLine("InnerExceptionMessage：" + ex.InnerException.Message);
                    _ErrorMsg.AppendLine("InnerExceptionStackTrace：" + ex.InnerException.StackTrace);
                }
            }
        }

        /// <summary>
        /// 将文件信息增加到线程队列中
        /// </summary>
        /// <param name="lstCActionEs"></param>
        public void AppendRangeCAction(List<CAction> lstCActionEs)
        {
            if (lstCActionEs == null || lstCActionEs.Count == 0)
            {
                return;
            }
            try
            {
                lock (_ObjListLock)
                {
                    if (_LstCActionEs == null)
                    {
                        _LstCActionEs = new List<CAction>();
                    }
                    int cntLstCAction = lstCActionEs.Count;
                    CAction[] arrCAction = new CAction[cntLstCAction];
                    lstCActionEs.CopyTo(arrCAction);
                    _LstCActionEs.AddRange(arrCAction);
                    _CntSumCAction += cntLstCAction;
                    _CntRemainderCAction += cntLstCAction;
                    //启动线程；
                    StartThread();
                }
            }
            catch (Exception ex)
            {
                _ErrorMsg.AppendLine("Message：" + ex.Message);
                _ErrorMsg.AppendLine("StackTrace：" + ex.StackTrace);
                if (ex.InnerException != null)
                {
                    _ErrorMsg.AppendLine("InnerExceptionMessage：" + ex.InnerException.Message);
                    _ErrorMsg.AppendLine("InnerExceptionStackTrace：" + ex.InnerException.StackTrace);
                }
            }
        }

        /// <summary>
        /// 启动线程
        /// </summary>
        private void StartThread()
        {
            //如果判断线程正在运行，则不进行启动操作；
            if (_BIsRunning)
            {
                return;
            }

            try
            {
                lock (_ObjListLock)
                {
                    //如果判断线程正在运行，则不进行启动操作；
                    if (_BIsRunning)
                    {
                        return;
                    }
                    //_PThread = new Thread(new ThreadStart(BatchAction));
                    _PThread = new Thread(BatchAction);
                    _PThread.IsBackground = true;
                    _PThread.SetApartmentState(ApartmentState.STA);
                    _PThread.Start();
                    _BIsRunning = true;
                }
            }
            catch (Exception ex)
            {
                //终止线程
                StopThread();
                //线程异常终止提示
                StartThreadFailAction();
                _ErrorMsg.AppendLine("Message：" + ex.Message);
                _ErrorMsg.AppendLine("StackTrace：" + ex.StackTrace);
                if (ex.InnerException != null)
                {
                    _ErrorMsg.AppendLine("InnerExceptionMessage：" + ex.InnerException.Message);
                    _ErrorMsg.AppendLine("InnerExceptionStackTrace：" + ex.InnerException.StackTrace);
                }

            }
        }

        /// <summary>
        /// 终止线程
        /// </summary>
        private void StopThread()
        {
            lock (_ObjListLock)
            {
                if (_LstCActionEs.Count > 0)
                {
                    StartThread();
                }
                //清空消息
                _UserMsg.Length = 0;
                _ErrorMsg.Length = 0;
                _NormalMsg.Length = 0;
                //停止线程状态
                _BIsRunning = false;
                _PThread = null;
                _LstCActionEs.Clear();
                //清空进度
                _CntSumCAction = 0;
                _CntRemainderCAction = 0;
                _IntCurrentCAction = 0;
                //垃圾回收
                GC.Collect();
            }
        }

        /// <summary>
        /// 批量执行
        /// </summary>
        private void BatchAction()
        {
            List<CAction> lstGetCActionEs = null;
            try
            {
                //从队列中取出CntBatchProcess个数据
                while ((lstGetCActionEs = GetCActionGroupByCount(_LstCActionEs, 0, CntBatchProcess)) != null)
                {
                    BatchActionDetail(lstGetCActionEs);
                    //执行操作后设置当前进度
                    //SetCurrentProcess();
                }
            }
            catch (Exception ex)
            {
                _ErrorMsg.AppendLine("Message：" + ex.Message);
                _ErrorMsg.AppendLine("StackTrace：" + ex.StackTrace);
                if (ex.InnerException != null)
                {
                    _ErrorMsg.AppendLine("InnerExceptionMessage：" + ex.InnerException.Message);
                    _ErrorMsg.AppendLine("InnerExceptionStackTrace：" + ex.InnerException.StackTrace);
                }
            }
            finally
            {
                string strNormalMsg =  _NormalMsg.ToString();
                string strErrorMsg = _ErrorMsg.ToString();
                int cntNotProcessedCActionEs = (lstGetCActionEs == null ? 0:lstGetCActionEs.Count) + _CntRemainderCAction;
                int cntSumCActionEs = _CntSumCAction;
                FinishAction();
                StopThread();
            }
        }

        /// <summary>
        /// 根据批量处理数据的数量，得到处理数据列表
        /// </summary>
        /// <param name="cntBatchProcess"></param>
        private List<CAction> GetCActionGroupByCount(List<CAction> lstCActionEs, int intStartIndex, int cntBatchProcess)
        {
            lock (_ObjListLock)
            {
                if (lstCActionEs == null ||
                    lstCActionEs.Count == 0 ||
                    intStartIndex < 0 || intStartIndex >= lstCActionEs.Count ||
                    cntBatchProcess <= 0)
                {
                    return null;
                }

                int intCurIndex = intStartIndex;
                int cntGetResult = 0;
                List<CAction> lstResults = new List<CAction>();
                while (intCurIndex < lstCActionEs.Count && cntGetResult < cntBatchProcess)
                {
                    lstResults.Add(lstCActionEs[intCurIndex]);
                    lstCActionEs.RemoveAt(intCurIndex);
                    cntGetResult++;
                }

                _IntCurrentCAction += cntGetResult;
                _CntRemainderCAction -= cntGetResult;
                return lstResults;
            }
        }

        /// <summary>
        /// 是否正在运行
        /// </summary>
        /// <returns></returns>
        public virtual bool IsRunning()
        {
            if (_PThread == null && !_BIsRunning)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 批量操作详细信息
        /// </summary>
        /// <param name="lstCActionEs"></param>
        public virtual void BatchActionDetail(List<CAction> lstCActionEs)
        {
        }

        /// <summary>
        /// 完成后执行
        /// </summary>
        public virtual void FinishAction()
        {
        }

        /// <summary>
        /// 线程启动失败之后操作
        /// </summary>
        public virtual void StartThreadFailAction()
        {
        }

        /// <summary>
        /// 得到当前进度值
        /// </summary>
        /// <returns></returns>
        public virtual int GetCurProcessValue()
        {
            return _IntCurrentCAction;
        }

        /// <summary>
        /// 得到当前最大值
        /// </summary>
        /// <returns></returns>
        public virtual int GetMaxValue()
        {
            return _CntSumCAction;
        }

        /// <summary>
        /// 得到当前任务状态
        /// </summary>
        /// <param name="strMessage"></param>
        /// <returns></returns>
        public ActionState GetCurrentActionState(out int cntSum,out int intDoing, out string strMessage)
        {
            cntSum = 0;
            intDoing = 0;
            strMessage =string.Empty;
            ActionState actionState = ActionState.None;

            cntSum = _CntSumCAction;
            intDoing = _IntCurrentCAction;
            if (_PThread == null || _LstCActionEs == null || _LstCActionEs.Count == 0)
            {
                return ActionState.Done;
            }
            else
            {
                return ActionState.Doing;
            }
        }

        /// <summary>
        /// 获取执行进度
        /// </summary>
        /// <returns></returns>
        public string GetExecuteProcessTip()
        {
            return string.Format("{0}进度提示：{1}/{2}", ModuleName, _IntCurrentCAction, _CntSumCAction);
        }
    }

    public enum ActionState
    {
        None,
        Doing,
        Done
    }
}
